package com.github.itzilly.sbt

import com.github.itzilly.sbt.features.RouteBuilder
import com.github.itzilly.sbt.features.mining.lockedChestESP.LockedChestEsp.highlightChests
import com.github.itzilly.sbt.features.mining.routeMaker.*
import com.github.itzilly.sbt.features.mining.routeMaker.RouteMaker.markerList
import com.github.itzilly.sbt.renderer.RenderUtils
import com.google.gson.Gson
import net.minecraft.client.Minecraft
import net.minecraft.util.AxisAlignedBB
import net.minecraft.util.ChatComponentText
import net.minecraftforge.common.MinecraftForge
import net.minecraftforge.fml.common.Mod
import net.minecraftforge.fml.common.event.FMLInitializationEvent
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import net.minecraftforge.fml.common.gameevent.InputEvent
import org.lwjgl.opengl.GL11
import java.awt.Color
import java.awt.Toolkit
import java.awt.datatransfer.Clipboard
import java.awt.datatransfer.StringSelection
import kotlin.math.round


@Mod(modid = "sbt", useMetadata = true)
class SkyBlockTweaks {
    init {
        INSTANCE = this
    }

    companion object {
        lateinit var INSTANCE: SkyBlockTweaks
    }

    @Mod.EventHandler
    fun init(event: FMLInitializationEvent) {
//        try {
//            val resource: net.minecraft.client.resources.IResource = Minecraft.getMinecraft().resourceManager
//                .getResource(net.minecraft.util.ResourceLocation("test:test.txt"))
//            org.apache.commons.io.IOUtils.copy(resource.inputStream, System.out)
//        } catch (e: java.io.IOException) {
//            throw java.lang.RuntimeException(e)
//        }

        // STBKeyBinds.init()

//        MinecraftForge.EVENT_BUS.register(this)
//        MinecraftForge.EVENT_BUS.register(KeyInputHandler())
//        MinecraftForge.EVENT_BUS.register(RouteMarkerRender)
//        MinecraftForge.EVENT_BUS.register(LockedChestEsp)

        MinecraftForge.EVENT_BUS.register(RouteBuilder)
    }
}

class KeyInputHandler {
    private val mc = Minecraft.getMinecraft()

    @SubscribeEvent
    fun onKeyInput(event: InputEvent.KeyInputEvent?) {
        if (STBKeyBinds.addWaypoint.isPressed) {
            val x = round(mc.thePlayer.posX - 0.5).toInt()
            val y = round(mc.thePlayer.posY).toInt()
            val z = round(mc.thePlayer.posZ - 0.5).toInt()

            if (!isPointInRouteList(markerList, x, y, z)) {
                val name = markerList.size + 1
                val point = RouteMarker(x, y, z, 0u, 1u, 0u, RouteOptions(name.toString()))
                markerList.add(point)
                mc.thePlayer.addChatMessage(ChatComponentText("Adding Point ($x, $y, $z) to route list"))
            } else {
                mc.thePlayer.addChatMessage(ChatComponentText("Point ($x, $y, $z) is already in the route list"))
            }

        } else if (STBKeyBinds.finishWaypoints.isPressed) {
            mc.thePlayer.addChatMessage(ChatComponentText("Finished route! Copied route to clipboard"))
            val jsonString = Gson().toJson(markerList)
            setClipboard(jsonString)
            markerList.clear()
        } else if (STBKeyBinds.clearWaypoints.isPressed) {
            markerList.clear()
            mc.thePlayer.addChatMessage(ChatComponentText("Reset current route"))
        } else if (STBKeyBinds.openRoutesGui.isPressed) {
            mc.displayGuiScreen(RouteGui())
        } else if (STBKeyBinds.xrayChests.isPressed) {
            highlightChests()
        }
    }
}


fun setClipboard(s: String) {
    val selection = StringSelection(s)
    val clipboard: Clipboard = Toolkit.getDefaultToolkit().systemClipboard
    clipboard.setContents(selection, selection)
}


class BlockRenderer {
    private var time = 0.0f

    fun update() {
        // Increment the time variable for animation
        time += 0.1f // Adjust the increment for desired speed
    }

    fun renderRouteMarker(marker: RouteMarker) {
        val box = AxisAlignedBB(
            marker.x.toDouble(),
            marker.y.toDouble(),
            marker.z.toDouble(),
            (marker.x + 1).toDouble(),
            (marker.y + 1).toDouble(),
            (marker.z + 1).toDouble()
        )
        val startColor = Color(marker.r.toInt() and 0xFF, marker.g.toInt() and 0xFF, marker.b.toInt() and 0xFF, 255)
        val endColor = Color(
            255 - (marker.r.toInt() and 0xFF),
            255 - (marker.g.toInt() and 0xFF),
            255 - (marker.b.toInt() and 0xFF),
            255
        )

        RenderUtils.drawAnimatedBlockTop(box, startColor, endColor, time)
        RenderUtils.drawAnimatedBlockBottom(box, startColor, endColor, time)
        RenderUtils.drawAnimatedBlockNorth(box, startColor, endColor, time)
        RenderUtils.drawAnimatedBlockEast(box, startColor, endColor, time)
        RenderUtils.drawAnimatedBlockSouth(box, startColor, endColor, time)
        RenderUtils.drawAnimatedBlockWest(box, startColor, endColor, time)
    }
}
