package com.github.itzilly.sbt.features.mining.lockedChestESP

import com.github.itzilly.sbt.features.RouteNode
import com.github.itzilly.sbt.features.mining.routeMaker.RouteMaker
import com.github.itzilly.sbt.features.mining.routeMaker.RouteMarker
import com.github.itzilly.sbt.renderer.RenderUtils
import com.github.itzilly.sbt.util.Vecd3
import net.minecraft.block.Block
import net.minecraft.client.Minecraft
import net.minecraft.client.renderer.GlStateManager
import net.minecraft.entity.Entity
import net.minecraft.util.AxisAlignedBB
import net.minecraft.util.BlockPos
import net.minecraftforge.client.event.RenderWorldLastEvent
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent
import org.lwjgl.opengl.GL11
import java.awt.Color


object ChestESPRender {
    @SubscribeEvent
    fun renderBlockOverlay(event: RenderWorldLastEvent) {
        val player = Minecraft.getMinecraft().thePlayer

        for (marker: RouteNode in LockedChestEsp.chestsList) {
            val pos = BlockPos(marker.x, marker.y, marker.z)
            renderPoint(player, pos, event.partialTicks)
            renderLine(pos, event.partialTicks)
        }
    }

    fun renderPoint(entity: Entity, blockPos: BlockPos, partialTicks: Float) {
        val padding: Double = 0.0020000000949949026

        val entityX: Double = entity.lastTickPosX + (entity.posX - entity.lastTickPosX) * partialTicks
        val entityY: Double = entity.lastTickPosY + (entity.posY - entity.lastTickPosY) * partialTicks
        val entityZ: Double = entity.lastTickPosZ + (entity.posZ - entity.lastTickPosZ) * partialTicks

        val outlineStartColor = 0xFFFFFF
        val outlineEndColor = 0xFFFFFF

        val world = Minecraft.getMinecraft().theWorld
        val blockState = world.getBlockState(blockPos)
        val block: Block = blockState.block

        // Create a default bounding box for blocks that don't have one
        val boundingBox: AxisAlignedBB = block.getCollisionBoundingBox(world, blockPos, blockState)
            ?: AxisAlignedBB(
                blockPos.x.toDouble(), blockPos.y.toDouble(), blockPos.z.toDouble(),
                blockPos.x + 1.0, blockPos.y + 1.0, blockPos.z + 1.0
            ).expand(padding, padding, padding)

        GL11.glPushMatrix()
        GlStateManager.disableTexture2D()
        GlStateManager.depthMask(false)
        GlStateManager.disableDepth()
        GL11.glEnable(GL11.GL_LINE_SMOOTH)
        GL11.glHint(GL11.GL_LINE_SMOOTH_HINT, GL11.GL_NICEST)
        GL11.glLineWidth(2.0f)
        GL11.glShadeModel(GL11.GL_SMOOTH)

        RenderUtils.drawBlock(boundingBox.offset(-entityX, -entityY, -entityZ), outlineStartColor, outlineEndColor)

        GL11.glLineWidth(2.0f)
        GL11.glDisable(GL11.GL_LINE_SMOOTH)
        GlStateManager.enableDepth()
        GlStateManager.depthMask(true)
        GlStateManager.enableAlpha()
        GlStateManager.enableTexture2D()
        GL11.glPopMatrix()
    }

    private fun renderLine(pos: BlockPos, partialTicks: Float) {
        val render = Minecraft.getMinecraft().renderViewEntity
        val rm = Minecraft.getMinecraft().renderManager
        // TODO: Fix this not working with view bobbing enabled
        val pos1 = Vecd3(rm.viewerPosX, rm.viewerPosY + render.eyeHeight, rm.viewerPosZ)

        val pos2 = Vecd3(
            pos.x.toDouble() + 0.5, // Center of the block
            pos.y.toDouble() + 0.5, // Center of the block
            pos.z.toDouble() + 0.5  // Center of the block
        )

        val lineColor = Color(0xe310d5)
        RenderUtils.drawLine(pos1, pos2, lineColor, 2, false, partialTicks)
    }

}