package com.github.itzilly.sbt.features.mining.lockedChestESP

import com.github.itzilly.sbt.features.NodeOptions
import com.github.itzilly.sbt.features.RouteBuilder
import com.github.itzilly.sbt.features.RouteNode
import com.github.itzilly.sbt.features.mining.routeMaker.RouteMaker
import com.github.itzilly.sbt.features.mining.routeMaker.RouteMarker
import com.github.itzilly.sbt.features.mining.routeMaker.RouteMarkerRender.renderPoint
import com.github.itzilly.sbt.features.mining.routeMaker.RouteOptions
import com.github.itzilly.sbt.util.DelayedFunction
import net.minecraft.client.Minecraft
import net.minecraft.init.Blocks
import net.minecraft.util.BlockPos
import net.minecraft.util.ChatComponentText
import net.minecraft.util.MovingObjectPosition
import net.minecraftforge.client.event.ClientChatReceivedEvent
import net.minecraftforge.client.event.RenderWorldLastEvent
import net.minecraftforge.event.entity.player.PlayerInteractEvent
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent

object LockedChestEsp {
    var chestsList: ArrayList<RouteNode> = ArrayList()
    
    @SubscribeEvent
    fun chatMessageEvent(event: ClientChatReceivedEvent) {
        if (event.message.unformattedText == "You uncovered a treasure chest!") {
            DelayedFunction({ highlightChests() }, 5)
        }
    }

    @SubscribeEvent
    fun renderBlockOverlay(event: RenderWorldLastEvent) {
        val player = Minecraft.getMinecraft().thePlayer

        for (marker: RouteMarker in RouteMaker.markerList) {
            val pos = BlockPos(marker.x, marker.y, marker.z)
            renderPoint(player, pos, event.partialTicks)
        }
    }

    @SubscribeEvent
    fun rightClickEvent(event: PlayerInteractEvent) {
        println("Event, $event")
        if (event.action == PlayerInteractEvent.Action.RIGHT_CLICK_BLOCK) {
            val player = event.entityPlayer
            val reachDistance = 5.0
            val eyePosition = player.getPositionEyes(1.0f)
            val lookVec = player.getLook(1.0f)
            val reachVec = eyePosition.addVector(
                lookVec.xCoord * reachDistance,
                lookVec.yCoord * reachDistance,
                lookVec.zCoord * reachDistance
            )
            val rayTraceResult = event.world.rayTraceBlocks(eyePosition, reachVec, false, false, false)

            if (rayTraceResult != null && rayTraceResult.typeOfHit == MovingObjectPosition.MovingObjectType.BLOCK) {
                val pos = rayTraceResult.blockPos

                val x = pos.x
                val y = pos.y
                val z = pos.z

                if (nodeExists(x, y, z)) {
                    chestsList.removeIf { it.x == x && it.y == y && it.z == z }
                }

            }
        }
    }

    fun nodeExists(x: Int, y: Int, z: Int): Boolean {
        return chestsList.any { it.x == x && it.y == y && it.z == z }
    }

    fun highlightChests() {
        val searchRadius = 8
        val world = Minecraft.getMinecraft().thePlayer.worldObj
        val posX = Minecraft.getMinecraft().thePlayer.posX
        val posY = Minecraft.getMinecraft().thePlayer.posY
        val posZ = Minecraft.getMinecraft().thePlayer.posZ

        val boundXMax = posX + searchRadius
        val boundXMin = posX - searchRadius

        val boundYMax = posY + searchRadius
        val boundYMin = posY - searchRadius

        val boundZMax = posZ + searchRadius
        val boundZMin = posZ - searchRadius

        val maxMarker =
            RouteMarker(boundXMax.toInt(), boundYMax.toInt(), boundZMax.toInt(), 1u, 0u, 0u, RouteOptions("Max Bounds"))
        val minMarker =
            RouteMarker(boundXMin.toInt(), boundYMin.toInt(), boundZMin.toInt(), 0u, 1u, 0u, RouteOptions("Min Bounds"))

        Minecraft.getMinecraft().thePlayer.addChatMessage(ChatComponentText("Searching..."))
        for (x in boundXMin.toInt()..boundXMax.toInt()) {
            for (y in boundYMin.toInt()..boundYMax.toInt()) {
                for (z in boundZMin.toInt()..boundZMax.toInt()) {
                    val block = world.getBlockState(BlockPos(x, y, z)).block
                    if (block == Blocks.chest || block == Blocks.trapped_chest || block == Blocks.ender_chest) {
                        val point = RouteNode(x, y, z, 0u, 0u, 1u, NodeOptions("Located point"))
                        chestsList.add(point)
                    }
                }
            }
        }
        Minecraft.getMinecraft().thePlayer.addChatMessage(ChatComponentText("Done!"))

//         SkyBlockTweaks.routeList.add(maxMarker)
//         SkyBlockTweaks.routeList.add(minMarker)
    }
}